package name.panitz.kotlin.date;
class Date(val day: Int, val month: Int, val year: Int) {

  val isLeapYear: Boolean
    get() = year % 400 == 0 || year % 4 == 0 && year % 100 != 0

  val absoluteDaysInYear: Int
    get() = if (isLeapYear) 366 else 365

  init {
    assert(month > 0)
    assert(month < 13)
    assert(day > 0)
    assert(day < 32)
  }

  fun nextDay(): Date {
    if (month == 12 && day == 31)
      return Date(1, 1, year + 1)
    return if (day == daysInMonth()) Date(1, month + 1, year) else Date(day + 1, month, year)
  }

  fun daysInMonth(): Int {
    if (month == 2)
      return if (isLeapYear) 29 else 28
    when (month) {
      4, 6, 9, 11 -> return 30
      else -> return 31
    }
  }

  fun monthAsHTML(): String {
    var result = "<table><tr><th>Mo</th><th>Di</th><th>Mi</th>" + "<th>Do</th><th>Fr</th><th>Sb</th><th>So</th></tr><tr>"
    var wd = Date(1, month, year).dayOfWeek()
    if (wd == 0)
      wd = 7
    for (i in 1 until wd) {
      result += "<td></td>"
    }
    var tds = wd - 1
    for (i in 1..daysInMonth()) {
      if (tds % 7 == 0 && i != 1) result += "</tr><tr>"
      result += "<td>$i</td>"
      tds++
    }
    result += "</tr>"
    return result + "</table>"
  }

  fun mothersDay(): Date {
    val wd = Date(1, 5, year).dayOfWeek()
    return Date(if (wd == 0) 8 else 15 - wd, 5, year)
  }

  fun dayOfWeek(): Int {
    // Schrittweise Lösung über die Einzelziffern

    val tz = day % 7
    val monatsziffern = intArrayOf(0, 3, 3, 6, 1, 4, 6, 2, 5, 0, 3, 5)
    val mz = monatsziffern[month - 1]
    val jz = year % 100
    val jahresziffer = (jz + jz / 4) % 7
    val jh = year / 100
    val jahrhundertziffer = (3 - jh % 4) * 2
    val schaltjahreskorrektur = if (isEarlierThan(Date(1, 3, year)) && isLeapYear) 6 else 0
    return (tz + mz + jahresziffer + jahrhundertziffer
        + schaltjahreskorrektur) % 7
    // Lösung mit Einsetzen direkt in die Formel
    // int y = (month < 3) ? (year - 1) : year;
    // return (day + (26 * ((month + 9) % 12 + 1) - 2) / 10 + y % 100 + (y % 100
    // / 4) + (y / 400) - 2 * (y / 100)) % 7;
  }

  fun easter(): Date {
    val a = year % 19
    val b = year / 100
    val c = year % 100
    val d = b / 4
    val e = b % 4
    val f = (b + 8) / 25
    val g = (b - f + 1) / 3
    val h = (19 * a + b - d - g + 15) % 30
    val i = c / 4
    val k = c % 4
    val l = (32 + 2 * e + 2 * i - h - k) % 7
    val m = (a + 11 * h + 22 * l) / 451
    val n = (h + l - 7 * m + 114) / 31
    val p = (h + l - 7 * m + 114) % 31

    return Date(p + 1, n, year)
  }


  fun isEarlierThan(that: Date): Boolean {
    return (year < that.year || year == that.year && month < that.month
        || year == that.year && month == that.month && day < that.day)
  }

  fun isLaterThan(that: Date): Boolean {
    return that.isEarlierThan(this)
  }

  fun isSameDate(that: Date): Boolean {
    return year == that.year && month == that.month && day == that.day
  }

  override fun toString(): String {
    val wds = arrayOf("So", "Mo", "Di", "Mi", "Do", "Fr", "Sb")
    return wds[dayOfWeek()] + " " + day + "." + month + "." + year
  }
}